/******************************* camera.c *******************************/

/* Provides an interface to the HP Photosmart R707 digital camera
   The camera has the following functions exposed to the micro:
   Input:  Power on sensing
   Output: On/Off
           Auto focus
           Shutter
           Video record
*/

#include <hidef.h> 
#include <mc9s12c32.h>

#include "camera.h"
#include "debug.h"
#include "timer.h"





// issues commands to the camera after each has had enough time to
// finish
unsigned char commandBuffer[MAX_COMMANDS];
unsigned char commandCurrent = 0;
int commandHead=0, commandLen=0, commandTail=0;



// waiting times for each action in units of clock ticks
// current values reflect a 24MHz clock
unsigned long cameraWaitTime[CAMERA_TOTAL_COMMANDS] = 
{
   0UL,        // null
   2400000UL,  // on
   2400000UL,  // off
   7200000UL,  // autofocus time, if it wasn't asserted before shutter
   2400000UL,  // shutter
   24000000UL,  // video start
   12000000UL,  // video stop
   24000000UL, // AF strobe to get initial focus lock
   4800000UL   // null strobe, turn everything off and wait
};
   

// time stamps for timing the events 
unsigned long cameraStartTime=0, cameraStopTime=0;



void Camera_Init(void) 
{
   // PTAD5 is input
   ATDDIEN |= CAMERA_MASK_POWERED;
   
   // PTAD5 has a pullup
   PERAD |= CAMERA_MASK_POWERED;
   PPSAD = 0b00000000;
   
   // PTT2-PTT5 output to camera
   DDRT |= CAMERA_MASK_ON | CAMERA_MASK_AF | CAMERA_MASK_SHUTTER | CAMERA_MASK_VIDEO;

   // Initialize event handler
   commandHead = 0;    // head of circular buffer, read from
   commandTail = 0;    // tail of circular buffer, written to
   commandLen = 0;     // number of items in circular buffer
   commandCurrent = 0; // current running command
}





// this function transitions the camera control pins
void Camera_HandleEvents(void)
{
   // handle current command
   if (commandCurrent != CAMERA_COMMAND_NONE) 
   {
      cameraStopTime = TimeStamp();
      
      if (cameraStopTime-cameraStartTime >= cameraWaitTime[commandCurrent])
      {
         // wait period is over, disassert signal
         switch(commandCurrent) 
         {
            case CAMERA_COMMAND_ON:
               CAMERA_UNSET_ON();
               break;
            case CAMERA_COMMAND_OFF:
               CAMERA_UNSET_ON();
               break;
            case CAMERA_COMMAND_AF:
               // do nothing, AF signal should remain on until shutter
               break;
            case CAMERA_COMMAND_SHUTTER:
               CAMERA_UNSET_SHUTTER();
               CAMERA_UNSET_AF();
               break;
            case CAMERA_COMMAND_VIDEO_START:
               CAMERA_UNSET_VIDEO();
               break;
            case CAMERA_COMMAND_VIDEO_STOP:
               CAMERA_UNSET_VIDEO();
               break;
            case CAMERA_COMMAND_AF_STROBE:
               CAMERA_UNSET_AF();
               break;
            case CAMERA_COMMAND_NULL_STROBE:
               // do nothing
               break;
         }
         
         // no current command now
         commandCurrent = CAMERA_COMMAND_NONE;
      }
   }
   
   // start next command if there is one
   if (commandLen > 0 && commandCurrent == CAMERA_COMMAND_NONE) 
   {
      // previously not running a command, now one is available
      
      if (commandBuffer[commandHead] == CAMERA_COMMAND_SHUTTER && 
          !(CAMERA_OUTPUT_PORT & CAMERA_MASK_AF))
      {
         // a shutter command was issued without an auto focus command before it
         // issue an AF command, and don't deque the shutter command
         commandCurrent = CAMERA_COMMAND_AF;
      }
      else 
      {
         // deque the next command
         commandCurrent = commandBuffer[commandHead];
         commandLen--;
         // circular increment
         commandHead++;
         if (commandHead == MAX_COMMANDS) commandHead=0;
      }
      
      cameraStartTime = TimeStamp();
      
      // raise the appropriate output
      switch(commandCurrent) 
      {
         case CAMERA_COMMAND_ON:
            if (!CAMERA_IS_POWERED())
            {                 
               CAMERA_SET_ON();
               DEBUG("Camera On\r\n");
            }
            else
            {
               // kill the command if camera already on
               commandCurrent=CAMERA_COMMAND_NONE;
            }           
            break;
         case CAMERA_COMMAND_OFF:
            if (CAMERA_IS_POWERED())
            {                 
               CAMERA_SET_ON();
               DEBUG("Camera Off\r\n");
            }
            else
            {
               // kill the command if camera already off
               commandCurrent=CAMERA_COMMAND_NONE;
            }              
            break;
         case CAMERA_COMMAND_AF:
            CAMERA_SET_AF();
            DEBUG("Camera Auto focus\r\n");
            break;
         case CAMERA_COMMAND_SHUTTER:
            CAMERA_SET_SHUTTER();
            DEBUG("Camera Shutter\r\n");
            break;
         case CAMERA_COMMAND_VIDEO_START:
            CAMERA_SET_VIDEO();
            DEBUG("Camera Video start\r\n");
            break;
         case CAMERA_COMMAND_VIDEO_STOP:
            CAMERA_SET_VIDEO();
            DEBUG("Camera Video stop\r\n");
            break;
         case CAMERA_COMMAND_AF_STROBE:
            CAMERA_SET_AF();
            DEBUG("Camera Auto focus strobe\r\n");
            break;
         case CAMERA_COMMAND_NULL_STROBE:
            // do nothing
            CAMERA_UNSET_ON();
            CAMERA_UNSET_AF();
            CAMERA_UNSET_SHUTTER();
            CAMERA_UNSET_VIDEO();
            DEBUG("Camera null strobe\r\n");
            break;
      }
               
   }
}

// a command to the camera queue
void Camera_Send(unsigned char command)
{
   if (command < CAMERA_TOTAL_COMMANDS) 
   {					      
      if (commandLen < MAX_COMMANDS)
      {
         // there is space in the buffer
         
         // enque the command
         commandBuffer[commandTail] = command;
         
         commandLen++;
         // circular increment
         commandTail++;
         if (commandTail == MAX_COMMANDS) commandTail=0;
         
      }
      else 
      {
         DEBUG("Not enough camera command buffer space\r\n");
      }
   }
   else 
   {
      DEBUG("Invalid camera command\r\n");
   }
}
