/******************************* radio.c *******************************/

// Uses Timer module to receive servo pulses
// Pulses are 1ms to 2ms every 20ms


// This driver is based upon measuring the number of clock cycles between a rising and falling edge
// However, the input capture registers are only 16bit.  This means that with a 24MHz clock, the
// maximum pulse width that can be measured is 2.73ms.  Exceeding that will lead cause the pulse
// width variables to wrap around to 0
 
#include <mc9s12c32.h>
#include "radio.h"
#include "timer.h"
#include "constants.h"

/* 9S12C32 Pins
   PTT6, PTT7
*/

// used for timing the incoming servo control pulses
unsigned short radio_start[RADIO_MAX_CHANNELS];
unsigned short radio_stop[RADIO_MAX_CHANNELS];
unsigned short radio_pulse[RADIO_MAX_CHANNELS];

// high for each channel to indicate new data
unsigned char radio_flag;

// configures the timer input capture to allow for receiving servo signals
void Radio_Init(void)
{					
   int i;
   
   // zero the counters
   for (i=0; i<RADIO_MAX_CHANNELS; i++)
   {
      radio_start[i]=0;
      radio_stop[i]=0;
      radio_pulse[i]=0;
   }
   
   
   // set PTT6,7 for input capture
   TIOS &= ~0b11000000;
   
   // input capture on rising edge
   TCTL3 = 0b01010000;
   
   // input capture on falling edge
   //TCTL3 = 0b10100000; 
   
   // set PTT7, PTT6 as input
   DDRT &= ~0b11000000;
   
   // enable interrupts on IOC7, IOC6
   TIE |= 0b11000000;

}


// interrupt handler for IOC7 channel
interrupt 15 void IOC7_Handler(void)  
{	    
   TFLG1 = 0x80;  // clear IOC7 interrupt flag
   
   if (TCTL3 & 0x80) // falling edge
   {
      // switch to rising edge
      TCTL3 = (TCTL3 & 0x3F) | 0x40;
      
      radio_stop[7] = TC7;      
      radio_pulse[7] = radio_stop[7] - radio_start[7];
      radio_flag |= 0x80;
   }
   else              // rising edge
   {
      // switch to falling edge
      TCTL3 = (TCTL3 & 0x3F) | 0x80;
      
      radio_start[7] = TC7;  
   }
}

// interrupt handler for IOC6 channel
interrupt 14 void IOC6_Handler(void)  
{
   TFLG1 = 0x40;  // clear IOC6 interrupt flag   
   
   if (TCTL3 & 0x20) // falling edge
   {
      // switch to rising edge
      TCTL3 = (TCTL3 & 0xCF) | 0x10;
      
      radio_stop[6] = TC6;
      radio_pulse[6] = radio_stop[6] - radio_start[6];
      radio_flag |= 0x40;
   }
   else              // rising edge
   {
      // switch to falling edge
      TCTL3 = (TCTL3 & 0xCF) | 0x20;
      
      radio_start[6] = TC6;   
   }
}


unsigned short Radio_PulseWidth(unsigned char channel)
{
   if (channel < RADIO_MAX_CHANNELS)
   {
      return radio_pulse[channel];
   }
   else
   {
      return -1;
   }
}

// returns true if the flag has be asserted, indicating a new recieved pulse
unsigned char Radio_ReadyFlag(unsigned char channel)
{
   if (channel < RADIO_MAX_CHANNELS)
   {        
      return (radio_flag & (1<<channel));
   }
   else
   {
      return 0;
   }
}

// acknowledges a flag by setting it low
void Radio_DisassertFlag(unsigned char channel)
{
   if (channel < RADIO_MAX_CHANNELS)
   {        
      radio_flag &= ~(1<<channel);
   }
}

